(*
                  umdlLPIIR
       (c) 2003 Paul C. L. Willmott, VP9MU

          Based on code from AO40RCV
         (c) 2000 Moe Wheatley AE4JY
             ae4jy@mindspring.com

 This program is free software; you can redistribute it
 and/or modify it under the terms of the GNU General
 Public License as published by the Free Software
 Foundation; either version 2 of the License, or at
 your option, any later version.

 This program is distributed in the hope that it will
 be useful, but WITHOUT ANY WARRANTY; without even the
 implied warranty of MERCHANTABILITY or FITNESS FOR A
 PARTICULAR PURPOSE.  See the GNU General Public
 License for more details.

 You should have received a copy of the GNU General
 Public License along with this program; if not, write
 to the Free Software Foundation, Inc., 59 Temple
 Place, Suite 330, Boston, MA  02111-1307  USA

            Contact : vp9mu@amsat.org
*)

(*
     Revision History
     ----------------

     1.0.0      Created July 21, 2003 PCLW
*)


//    A 3rd order LP complex IIR section.

unit umdlLPIIR;

interface

type

   TComplex = record
                    x : double ;
                    y : double ;
              end ;

   TLPCoef = array[0..8-1] of double ;

   TLPIIR = class(TObject)
   private
      pW1 : TComplex ;
      dIIR1_A1 : double ;
      dIIR1_B0 : double ;
      dIIR1_B1 : double ;
      pW2 : array[0..2-1] of TComplex ;
      dIIR2_A1 : double ;
      dIIR2_A2 : double ;
      dIIR2_B0 : double ;
      dIIR2_B1 : double ;
      dIIR2_B2 : double ;

   public
      function CalcFilter(pIn:TComplex) : TComplex ;
      function IIRFilter1(pIn:TComplex) : TComplex ;
      function IIRFilter2(pIn:TComplex) : TComplex ;
      procedure Init(pLPCoef:TLPCoef) ;
   end ;

implementation

{ TLPIIR }

function TLPIIR.CalcFilter(pIn:TComplex) : TComplex ;
//  Calculate complex 3rd order IIR LP filter
begin
     result := IIRFilter2(IIRFilter1(pIn)) ;
end { TLPIIR.CalcFilter } ;

function TLPIIR.IIRFilter1(pIn:TComplex) : TComplex ;
(*==================================================================*/
/*  Function: IIRFilter1                                            */
/*                                                                  */
/*  Implements a first order IIR filter stage using canonical form. */
/*  The transfer function of the filter stage implemented is in     */
/*  the form                                                        */
/*                                                                  */
/*                          -1                                      */
/*                 B0 + B1 z                                        */
/*          H(z) = -----------                                      */
/*                         -1                                       */
/*                  1 + A1 z                                        */
/*                                                                  */
/*  The block diagram used in the implementation is given below:    */
/*                                                                  */
/*          input               w(n)   B0         output            */
/*            -----> + ----------+-----[>--> + ----->               */
/*                   |           |           |                      */
/*                   |        +--+--+        |                      */
/*                   |        |Delay|        |                      */
/*                   |        +--+--+        |                      */
/*                   |   -A1     |w(n-1) B1  |                      */
/*                   +----<]-----+-------[>--+                      */
/*                                                                  */
/*==================================================================*)

var
     w, pOut : TComplex ;

begin
     w.x := pIn.x - (dIIR1_A1 * pW1.x) ;
     w.y := pIn.y - (dIIR1_A1 * pW1.y) ;
     pOut.x := dIIR1_B0 * w.x + dIIR1_B1 * pW1.x ;
     pOut.y := dIIR1_B0 * w.y + dIIR1_B1 * pW1.y ;
     pW1 := w ;
     result := pOut ;
end { TLPIIR.IIRFilter1 } ;

function TLPIIR.IIRFilter2(pIn:TComplex) : TComplex ;
(*==================================================================*/
/*  Function: IIRFilter2                                            */
/*                                                                  */
/*  Implements a second order IIR filter stage in canonical form.   */
/*  The transfer function of the filter stage implemented is in     */
/*  the form                                                        */
/*                                                                  */
/*                          -1       -2                             */
/*                 B0 + B1 z   + B2 z                               */
/*          H(z) = --------------------                             */
/*                         -1       -2                              */
/*                 1 + A1 z   + A2 z                                */
/*                                                                  */
/*  The block diagram used in the implementation is given below:    */
/*                                                                  */
/*          input               w(n)   B0         output            */
/*            -----> + ----------+-----[>--> + ----->               */
/*                   |           |           |                      */
/*                   |        +--+--+        |                      */
/*                   |        |Delay|        |                      */
/*                   |        +--+--+        |                      */
/*                   |   -A1     |w(n-1) B1  |                      */
/*                   +----<]-----+-------[>--+                      */
/*                   |           |           |                      */
/*                   |        +--+--+        |                      */
/*                   |        |Delay|        |                      */
/*                   |        +--+--+        |                      */
/*                   |   -A2     |w(n-2) B2  |                      */
/*                   +----<]-----+-------[>--+                      */
/*                                                                  */
/*==================================================================*)
var
     w, pOut : TComplex ;

begin

     w.x  := pIn.x - dIIR2_A1*pW2[0].x - dIIR2_A2*pW2[1].x ;
     w.y  := pIn.y - dIIR2_A1*pW2[0].y - dIIR2_A2*pW2[1].y ;
     pOut.x := dIIR2_B0*w.x + dIIR2_B1*pW2[0].x + dIIR2_B2*pW2[1].x ;
     pOut.y := dIIR2_B0*w.y + dIIR2_B1*pW2[0].y + dIIR2_B2*pW2[1].y ;
     pW2[1] := pW2[0] ;
     pW2[0] := w ;
     result := pOut ;

end { TLPIIR.IIRFilter2 } ;

procedure TLPIIR.Init(pLPCoef:TLPCoef);
//////////////////////////////////////////////////////////////////////
//  Sets the filter coefficients depending on filter number.
//
//   B1, B0, A1   for the 1st order section
//
//   B2, B1, B0, A2, A1  for the 2nd order section
//
//////////////////////////////////////////////////////////////////////
begin

     dIIR1_B1 := pLPCoef[0] ;
     dIIR1_B0 := pLPCoef[1] ;
     dIIR1_A1 := pLPCoef[2] ;
//
     dIIR2_B2 := pLPCoef[3] ;
     dIIR2_B1 := pLPCoef[4] ;
     dIIR2_B0 := pLPCoef[5] ;
     dIIR2_A2 := pLPCoef[6] ;
     dIIR2_A1 := pLPCoef[7] ;
//
     pW1.x := 0.0 ;
     pW1.y := 0.0 ;
//
     pW2[0].x := 0.0 ;
     pW2[0].y := 0.0 ;
     pW2[1].x := 0.0 ;
     pW2[1].y := 0.0 ;

end { TLPIIR.Init } ;

end.
